@@ -49,15 +49,12 @@ enum MP4PropertyType {
 
 class MP4Property {
 public:
-    MP4Property(const char *name = NULL);
+    MP4Property(MP4Atom& parentAtom, const char *name = NULL);
 
     virtual ~MP4Property() { }
 
-    MP4Atom* GetParentAtom() {
-        return m_pParentAtom;
-    }
-    virtual void SetParentAtom(MP4Atom* pParentAtom) {
-        m_pParentAtom = pParentAtom;
+    MP4Atom& GetParentAtom() {
+        return m_parentAtom;
     }
 
     const char *GetName() {
@@ -85,29 +82,34 @@ public:
 
     virtual void Generate() { /* default is a no-op */ };
 
-    virtual void Read(MP4File* pFile, uint32_t index = 0) = 0;
+    virtual void Read(MP4File& file, uint32_t index = 0) = 0;
 
-    virtual void Write(MP4File* pFile, uint32_t index = 0) = 0;
+    virtual void Write(MP4File& file, uint32_t index = 0) = 0;
 
-    virtual void Dump(FILE* pFile, uint8_t indent,
+    virtual void Dump(uint8_t indent,
                       bool dumpImplicits, uint32_t index = 0) = 0;
 
     virtual bool FindProperty(const char* name,
                               MP4Property** ppProperty, uint32_t* pIndex = NULL);
 
 protected:
-    MP4Atom* m_pParentAtom;
+    MP4Atom& m_parentAtom;
     const char* m_name;
     bool m_readOnly;
     bool m_implicit;
+
+private:
+    MP4Property();
+    MP4Property ( const MP4Property &src );
+    MP4Property &operator= ( const MP4Property &src );
 };
 
 MP4ARRAY_DECL(MP4Property, MP4Property*);
 
 class MP4IntegerProperty : public MP4Property {
 protected:
-    MP4IntegerProperty(const char* name)
-            : MP4Property(name) { };
+    MP4IntegerProperty(MP4Atom& parentAtom, const char* name)
+            : MP4Property(parentAtom, name) { };
 
 public:
     uint64_t GetValue(uint32_t index = 0);
@@ -119,13 +121,18 @@ public:
     void DeleteValue(uint32_t index = 0);
 
     void IncrementValue(int32_t increment = 1, uint32_t index = 0);
+
+private:
+    MP4IntegerProperty();
+    MP4IntegerProperty ( const MP4IntegerProperty &src );
+    MP4IntegerProperty &operator= ( const MP4IntegerProperty &src );
 };
 
 #define MP4INTEGER_PROPERTY_DECL2(isize, xsize) \
     class MP4Integer##xsize##Property : public MP4IntegerProperty { \
     public: \
-        MP4Integer##xsize##Property(const char* name) \
-            : MP4IntegerProperty(name) { \
+        MP4Integer##xsize##Property(MP4Atom& parentAtom, const char* name) \
+            : MP4IntegerProperty(parentAtom, name) { \
             SetCount(1); \
             m_values[0] = 0; \
         } \
@@ -147,7 +154,9 @@ public:
         \
         void SetValue(uint##isize##_t value, uint32_t index = 0) { \
             if (m_readOnly) { \
-                throw new MP4Error(EACCES, "property is read-only", m_name); \
+                ostringstream msg; \
+                msg << "property is read-only: " << m_name; \
+                throw new PlatformException(msg.str().c_str(), EACCES, __FILE__, __LINE__, __FUNCTION__); \
             } \
             m_values[index] = value; \
         } \
@@ -163,24 +172,28 @@ public:
         void IncrementValue(int32_t increment = 1, uint32_t index = 0) { \
             m_values[index] += increment; \
         } \
-        void Read(MP4File* pFile, uint32_t index = 0) { \
+        void Read(MP4File& file, uint32_t index = 0) { \
             if (m_implicit) { \
                 return; \
             } \
-            m_values[index] = pFile->ReadUInt##xsize(); \
+            m_values[index] = file.ReadUInt##xsize(); \
         } \
         \
-        void Write(MP4File* pFile, uint32_t index = 0) { \
+        void Write(MP4File& file, uint32_t index = 0) { \
             if (m_implicit) { \
                 return; \
             } \
-            pFile->WriteUInt##xsize(m_values[index]); \
+            file.WriteUInt##xsize(m_values[index]); \
         } \
-        void Dump(FILE* pFile, uint8_t indent, \
+        void Dump(uint8_t indent, \
             bool dumpImplicits, uint32_t index = 0); \
     \
     protected: \
         MP4Integer##isize##Array m_values; \
+    private: \
+        MP4Integer##xsize##Property(); \
+        MP4Integer##xsize##Property ( const MP4Integer##xsize##Property &src ); \
+        MP4Integer##xsize##Property &operator= ( const MP4Integer##xsize##Property &src ); \
     };
 
 #define MP4INTEGER_PROPERTY_DECL(size) \
@@ -194,8 +207,8 @@ MP4INTEGER_PROPERTY_DECL(64);
 
 class MP4BitfieldProperty : public MP4Integer64Property {
 public:
-    MP4BitfieldProperty(const char* name, uint8_t numBits)
-            : MP4Integer64Property(name) {
+    MP4BitfieldProperty(MP4Atom& parentAtom, const char* name, uint8_t numBits)
+            : MP4Integer64Property(parentAtom, name) {
         ASSERT(numBits != 0);
         ASSERT(numBits <= 64);
         m_numBits = numBits;
@@ -208,19 +221,24 @@ public:
         m_numBits = numBits;
     }
 
-    void Read(MP4File* pFile, uint32_t index = 0);
-    void Write(MP4File* pFile, uint32_t index = 0);
-    void Dump(FILE* pFile, uint8_t indent,
+    void Read(MP4File& file, uint32_t index = 0);
+    void Write(MP4File& file, uint32_t index = 0);
+    void Dump(uint8_t indent,
               bool dumpImplicits, uint32_t index = 0);
 
 protected:
     uint8_t m_numBits;
+
+private:
+    MP4BitfieldProperty();
+    MP4BitfieldProperty ( const MP4BitfieldProperty &src );
+    MP4BitfieldProperty &operator= ( const MP4BitfieldProperty &src );
 };
 
 class MP4Float32Property : public MP4Property {
 public:
-    MP4Float32Property(const char* name)
-            : MP4Property(name) {
+    MP4Float32Property(MP4Atom& parentAtom, const char* name)
+            : MP4Property(parentAtom, name) {
         m_useFixed16Format = false;
         m_useFixed32Format = false;
         SetCount(1);
@@ -244,7 +262,9 @@ public:
 
     void SetValue(float value, uint32_t index = 0) {
         if (m_readOnly) {
-            throw new MP4Error(EACCES, "property is read-only", m_name);
+            ostringstream msg;
+            msg << "property is read-only: " << m_name;
+            throw new PlatformException(msg.str().c_str(), EACCES, __FILE__, __LINE__, __FUNCTION__);
         }
         m_values[index] = value;
     }
@@ -273,20 +293,25 @@ public:
         m_useFixed32Format = useFixed32Format;
     }
 
-    void Read(MP4File* pFile, uint32_t index = 0);
-    void Write(MP4File* pFile, uint32_t index = 0);
-    void Dump(FILE* pFile, uint8_t indent,
+    void Read(MP4File& file, uint32_t index = 0);
+    void Write(MP4File& file, uint32_t index = 0);
+    void Dump(uint8_t indent,
               bool dumpImplicits, uint32_t index = 0);
 
 protected:
     bool m_useFixed16Format;
     bool m_useFixed32Format;
     MP4Float32Array m_values;
+
+private:
+    MP4Float32Property();
+    MP4Float32Property ( const MP4Float32Property &src );
+    MP4Float32Property &operator= ( const MP4Float32Property &src );
 };
 
 class MP4StringProperty : public MP4Property {
 public:
-    MP4StringProperty(const char* name,
+    MP4StringProperty(MP4Atom& parentAtom, const char* name,
                       bool useCountedFormat = false, bool useUnicode = false, bool arrayMode = false);
 
     ~MP4StringProperty();
@@ -345,9 +370,9 @@ public:
         m_fixedLength = fixedLength;
     }
 
-    void Read(MP4File* pFile, uint32_t index = 0);
-    void Write(MP4File* pFile, uint32_t index = 0);
-    void Dump(FILE* pFile, uint8_t indent,
+    void Read(MP4File& file, uint32_t index = 0);
+    void Write(MP4File& file, uint32_t index = 0);
+    void Dump(uint8_t indent,
               bool dumpImplicits, uint32_t index = 0);
 
 protected:
@@ -358,11 +383,16 @@ protected:
     uint32_t m_fixedLength;
 
     MP4StringArray m_values;
+
+private:
+    MP4StringProperty();
+    MP4StringProperty ( const MP4StringProperty &src );
+    MP4StringProperty &operator= ( const MP4StringProperty &src );
 };
 
 class MP4BytesProperty : public MP4Property {
 public:
-    MP4BytesProperty(const char* name, uint32_t valueSize = 0,
+    MP4BytesProperty(MP4Atom& parentAtom, const char* name, uint32_t valueSize = 0,
                      uint32_t defaultValueSize = 0);
 
     ~MP4BytesProperty();
@@ -385,6 +415,17 @@ public:
         *pValueSize = m_valueSizes[index];
     }
 
+    char* GetValueStringAlloc( uint32_t index = 0 ) {
+        char* buf = (char*)MP4Malloc( m_valueSizes[index] + 1 );
+        memcpy( buf, m_values[index], m_valueSizes[index] );
+        buf[m_valueSizes[index]] = '\0';
+        return buf;
+    }
+
+    bool CompareToString( const string& s, uint32_t index = 0 ) {
+        return string( (const char*)m_values[index], m_valueSizes[index] ) != s;
+    }
+
     void CopyValue(uint8_t* pValue, uint32_t index = 0) {
         // N.B. caller takes responsbility for valid pointer
         // and sufficient memory at the destination
@@ -400,7 +441,7 @@ public:
         SetValue(pValue, valueSize, count);
     }
 
-    uint32_t GetValueSize(uint32_t valueSize, uint32_t index = 0) {
+    uint32_t GetValueSize( uint32_t index = 0 ) {
         return m_valueSizes[index];
     }
 
@@ -412,9 +453,9 @@ public:
 
     void SetFixedSize(uint32_t fixedSize);
 
-    void Read(MP4File* pFile, uint32_t index = 0);
-    void Write(MP4File* pFile, uint32_t index = 0);
-    void Dump(FILE* pFile, uint8_t indent,
+    void Read(MP4File& file, uint32_t index = 0);
+    void Write(MP4File& file, uint32_t index = 0);
+    void Dump(uint8_t indent,
               bool dumpImplicits, uint32_t index = 0);
 
 protected:
@@ -422,11 +463,16 @@ protected:
     uint32_t        m_defaultValueSize;
     MP4Integer32Array   m_valueSizes;
     MP4BytesArray       m_values;
+
+private:
+    MP4BytesProperty();
+    MP4BytesProperty ( const MP4BytesProperty &src );
+    MP4BytesProperty &operator= ( const MP4BytesProperty &src );
 };
 
 class MP4TableProperty : public MP4Property {
 public:
-    MP4TableProperty(const char* name, MP4IntegerProperty* pCountProperty);
+    MP4TableProperty(MP4Atom& parentAtom, const char* name, MP4IntegerProperty* pCountProperty);
 
     ~MP4TableProperty();
 
@@ -434,13 +480,6 @@ public:
         return TableProperty;
     }
 
-    void SetParentAtom(MP4Atom* pParentAtom) {
-        m_pParentAtom = pParentAtom;
-        for (uint32_t i = 0; i < m_pProperties.Size(); i++) {
-            m_pProperties[i]->SetParentAtom(pParentAtom);
-        }
-    }
-
     void AddProperty(MP4Property* pProperty);
 
     MP4Property* GetProperty(uint32_t index) {
@@ -454,17 +493,17 @@ public:
         m_pCountProperty->SetValue(count);
     }
 
-    void Read(MP4File* pFile, uint32_t index = 0);
-    void Write(MP4File* pFile, uint32_t index = 0);
-    void Dump(FILE* pFile, uint8_t indent,
+    void Read(MP4File& file, uint32_t index = 0);
+    void Write(MP4File& file, uint32_t index = 0);
+    void Dump(uint8_t indent,
               bool dumpImplicits, uint32_t index = 0);
 
     bool FindProperty(const char* name,
                       MP4Property** ppProperty, uint32_t* pIndex = NULL);
 
 protected:
-    virtual void ReadEntry(MP4File* pFile, uint32_t index);
-    virtual void WriteEntry(MP4File* pFile, uint32_t index);
+    virtual void ReadEntry(MP4File& file, uint32_t index);
+    virtual void WriteEntry(MP4File& file, uint32_t index);
 
     bool FindContainedProperty(const char* name,
                                MP4Property** ppProperty, uint32_t* pIndex);
@@ -472,11 +511,16 @@ protected:
 protected:
     MP4IntegerProperty* m_pCountProperty;
     MP4PropertyArray    m_pProperties;
+
+private:
+    MP4TableProperty();
+    MP4TableProperty ( const MP4TableProperty &src );
+    MP4TableProperty &operator= ( const MP4TableProperty &src );
 };
 
 class MP4DescriptorProperty : public MP4Property {
 public:
-    MP4DescriptorProperty(const char* name = NULL,
+    MP4DescriptorProperty(MP4Atom& parentAtom, const char* name = NULL,
                           uint8_t tagsStart = 0, uint8_t tagsEnd = 0,
                           bool mandatory = false, bool onlyOne = false);
 
@@ -513,16 +557,16 @@ public:
     void DeleteDescriptor(uint32_t index);
 
     void Generate();
-    void Read(MP4File* pFile, uint32_t index = 0);
-    void Write(MP4File* pFile, uint32_t index = 0);
-    void Dump(FILE* pFile, uint8_t indent,
+    void Read(MP4File& file, uint32_t index = 0);
+    void Write(MP4File& file, uint32_t index = 0);
+    void Dump(uint8_t indent,
               bool dumpImplicits, uint32_t index = 0);
 
     bool FindProperty(const char* name,
                       MP4Property** ppProperty, uint32_t* pIndex = NULL);
 
 protected:
-    virtual MP4Descriptor* CreateDescriptor(uint8_t tag);
+    virtual MP4Descriptor* CreateDescriptor(MP4Atom& parentAtom, uint8_t tag);
 
     bool FindContainedProperty(const char* name,
                                MP4Property** ppProperty, uint32_t* pIndex);
@@ -534,22 +578,32 @@ protected:
     bool                m_mandatory;
     bool                m_onlyOne;
     MP4DescriptorArray  m_pDescriptors;
+
+private:
+    MP4DescriptorProperty();
+    MP4DescriptorProperty ( const MP4DescriptorProperty &src );
+    MP4DescriptorProperty &operator= ( const MP4DescriptorProperty &src );
 };
 
 class MP4QosQualifierProperty : public MP4DescriptorProperty {
 public:
-    MP4QosQualifierProperty(const char* name = NULL,
+    MP4QosQualifierProperty(MP4Atom& parentAtom, const char* name = NULL,
                             uint8_t tagsStart = 0, uint8_t tagsEnd = 0,
                             bool mandatory = false, bool onlyOne = false) :
-            MP4DescriptorProperty(name, tagsStart, tagsEnd, mandatory, onlyOne) { }
+            MP4DescriptorProperty(parentAtom, name, tagsStart, tagsEnd, mandatory, onlyOne) { }
 
 protected:
-    MP4Descriptor* CreateDescriptor(uint8_t tag);
+    MP4Descriptor* CreateDescriptor(MP4Atom& parentAtom, uint8_t tag);
+
+private:
+    MP4QosQualifierProperty();
+    MP4QosQualifierProperty ( const MP4QosQualifierProperty &src );
+    MP4QosQualifierProperty &operator= ( const MP4QosQualifierProperty &src );
 };
 
 ///////////////////////////////////////////////////////////////////////////////
 
-/// ISO-639-2/T langauge code.
+/// ISO-639-2/T language code.
 /// Language codes are 3-alpha (always lowercase) codes which are then
 /// offset using 0x60 and packed as 5-bit values into 16-bits, most
 /// significant bit is zero-padding.
@@ -559,18 +613,23 @@ private:
     bmff::LanguageCode _value;
 
 public:
-    explicit MP4LanguageCodeProperty( const char* , bmff::LanguageCode = bmff::ILC_UND );
-    MP4LanguageCodeProperty( const char* , const string& );
+    explicit MP4LanguageCodeProperty( MP4Atom& parentAtom, const char* , bmff::LanguageCode = bmff::ILC_UND );
+    MP4LanguageCodeProperty( MP4Atom& parentAtom, const char* , const string& );
 
     MP4PropertyType GetType();
     uint32_t        GetCount();
     void            SetCount( uint32_t );
-    void            Read( MP4File*, uint32_t = 0 );
-    void            Write( MP4File*, uint32_t = 0 );
-    void            Dump( FILE*, uint8_t, bool, uint32_t = 0 );
+    void            Read( MP4File&, uint32_t = 0 );
+    void            Write( MP4File&, uint32_t = 0 );
+    void            Dump( uint8_t, bool, uint32_t = 0 );
 
     bmff::LanguageCode GetValue();
     void               SetValue( bmff::LanguageCode );
+
+private:
+    MP4LanguageCodeProperty();
+    MP4LanguageCodeProperty ( const MP4LanguageCodeProperty &src );
+    MP4LanguageCodeProperty &operator= ( const MP4LanguageCodeProperty &src );
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -580,16 +639,21 @@ private:
     itmf::BasicType _value;
 
 public:
-    explicit MP4BasicTypeProperty( const char* , itmf::BasicType = itmf::BT_UNDEFINED );
+    explicit MP4BasicTypeProperty( MP4Atom& parentAtom, const char* , itmf::BasicType = itmf::BT_UNDEFINED );
 
     MP4PropertyType GetType();
     uint32_t        GetCount();
     void            SetCount( uint32_t );
-    void            Read( MP4File*, uint32_t = 0 );
-    void            Write( MP4File*, uint32_t = 0 );
-    void            Dump( FILE*, uint8_t, bool, uint32_t = 0 );
+    void            Read( MP4File&, uint32_t = 0 );
+    void            Write( MP4File&, uint32_t = 0 );
+    void            Dump( uint8_t, bool, uint32_t = 0 );
     itmf::BasicType GetValue();
     void            SetValue( itmf::BasicType );
+
+private:
+    MP4BasicTypeProperty();
+    MP4BasicTypeProperty ( const MP4BasicTypeProperty &src );
+    MP4BasicTypeProperty &operator= ( const MP4BasicTypeProperty &src );
 };
 
 ///////////////////////////////////////////////////////////////////////////////
